'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var lastId = 0;
var registry = {};

var guid = function guid() {
  return lastId++;
};

var registerStyle = function registerStyle(style) {
  var id = guid();
  registry[id] = style;
  return id;
};

var resolveStyle = function resolveStyle(id) {
  return registry[id];
};

var create = function create(styles) {
  var result = {};
  Object.keys(styles).forEach(function (key) {
    result[key] = registerStyle(styles[key]);
  });
  return result;
};

var merge = function merge() {
  var left = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var right = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return _extends({}, left, right);
};

var flatten = function flatten(input) {
  if (Array.isArray(input)) {
    return input.reduce(function (acc, val) {
      return merge(acc, flatten(val));
    }, {});
  } else if (typeof input === 'number') {
    return resolveStyle(input);
  } else if (!input) {
    return undefined;
  }
  return input;
};

var resolve = function resolve(style) {
  return flatten(style);
};

var StyleSheet = {
  hairlineWidth: 1,
  absoluteFill: registerStyle({
    position: 'absolute',
    top: 0,
    left: 0,
    bottom: 0,
    right: 0
  }),
  create: create,
  flatten: flatten,
  resolve: resolve
};

exports.default = StyleSheet;